@@ -1,6 +1,5 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Big = imports.gi.Big;
 const Clutter = imports.gi.Clutter;
 const Gtk = imports.gi.Gtk;
 const Mainloop = imports.mainloop;
@@ -18,16 +17,13 @@ const DocDisplay = imports.ui.docDisplay;
 const PlaceDisplay = imports.ui.placeDisplay;
 const GenericDisplay = imports.ui.genericDisplay;
 const Main = imports.ui.main;
+const Overview = imports.ui.overview;
 const Search = imports.ui.search;
+const Tweener = imports.ui.tweener;
 
 // 25 search results (per result type) should be enough for everyone
 const MAX_RENDERED_SEARCH_RESULTS = 25;
 
-const DEFAULT_PADDING = 4;
-
-const DOCS = "docs";
-const PLACES = "places";
-
 /*
  * Returns the index in an array of a given length that is obtained
  * if the provided index is incremented by an increment and the array
@@ -41,14 +37,6 @@ function _getIndexWrapped(index, increment, length) {
    return (index + increment + length) % length;
 }
 
-function _createDisplay(displayType, flags) {
-    if (displayType == DOCS)
-        return new DocDisplay.DocDisplay(flags);
-    else if (displayType == PLACES)
-        return new PlaceDisplay.PlaceDisplay(flags);
-    return null;
-}
-
 function Pane() {
     this._init();
 }
@@ -57,7 +45,7 @@ Pane.prototype = {
     _init: function () {
         this._open = false;
 
-        this.actor = new St.BoxLayout({ style_class: "dash-pane",
+        this.actor = new St.BoxLayout({ style_class: 'dash-pane',
                                          vertical: true,
                                          reactive: true });
         this.actor.connect('button-press-event', Lang.bind(this, function (a, e) {
@@ -65,15 +53,6 @@ Pane.prototype = {
             return true;
         }));
 
-        let chromeTop = new St.BoxLayout();
-
-        let dummy = new St.Bin();
-        chromeTop.add(dummy, { expand: true });
-        this.actor.add(chromeTop);
-
-        this.content = new St.BoxLayout({ vertical: true });
-        this.actor.add(this.content, { expand: true });
-
         // Hidden by default
         this.actor.hide();
     },
@@ -82,20 +61,33 @@ Pane.prototype = {
         if (this._open)
             return;
         this._open = true;
-        this.actor.show();
         this.emit('open-state-changed', this._open);
+        this.actor.opacity = 0;
+        this.actor.show();
+        Tweener.addTween(this.actor,
+                         { opacity: 255,
+                           time: Overview.PANE_FADE_TIME,
+                           transition: 'easeOutQuad'
+                         });
     },
 
     close: function () {
         if (!this._open)
             return;
         this._open = false;
-        this.actor.hide();
-        this.emit('open-state-changed', this._open);
+        Tweener.addTween(this.actor,
+                         { opacity: 0,
+                           time: Overview.PANE_FADE_TIME,
+                           transition: 'easeOutQuad',
+                           onComplete: Lang.bind(this, function() {
+                               this.actor.hide();
+                               this.emit('open-state-changed', this._open);
+                           })
+                         });
     },
 
     destroyContent: function() {
-        let children = this.content.get_children();
+        let children = this.actor.get_children();
         for (let i = 0; i < children.length; i++) {
             children[i].destroy();
         }
@@ -110,53 +102,22 @@ Pane.prototype = {
 };
 Signals.addSignalMethods(Pane.prototype);
 
-function ResultArea(displayType, flags) {
-    this._init(displayType, flags);
+function ResultArea() {
+    this._init();
 }
 
 ResultArea.prototype = {
-    _init : function(displayType, flags) {
-        this.actor = new Big.Box({ orientation: Big.BoxOrientation.VERTICAL });
-        this.resultsContainer = new Big.Box({ orientation: Big.BoxOrientation.HORIZONTAL,
-                                              spacing: DEFAULT_PADDING
-                                            });
-        this.actor.append(this.resultsContainer, Big.BoxPackFlags.EXPAND);
+    _init : function() {
+        this.actor = new St.BoxLayout({ vertical: true });
+        this.resultsContainer = new St.BoxLayout({ style_class: 'dash-results-container' });
+        this.actor.add(this.resultsContainer, { expand: true });
 
-        this.display = _createDisplay(displayType, flags);
-        this.resultsContainer.append(this.display.actor, Big.BoxPackFlags.EXPAND);
+        this.display = new DocDisplay.DocDisplay();
+        this.resultsContainer.add(this.display.actor, { expand: true });
         this.display.load();
     }
 };
 
-// Utility function shared between ResultPane and the DocDisplay in the main dash.
-// Connects to the detail signal of the display, and on-demand creates a new
-// pane.
-function createPaneForDetails(dash, display) {
-    let detailPane = null;
-    display.connect('show-details', Lang.bind(this, function(display, index) {
-        if (detailPane == null) {
-            detailPane = new Pane();
-            detailPane.connect('open-state-changed', Lang.bind(this, function (pane, isOpen) {
-                if (!isOpen) {
-                    /* Ensure we don't keep around large preview textures */
-                    detailPane.destroyContent();
-                }
-            }));
-            dash._addPane(detailPane, St.Align.START);
-        }
-
-        if (index >= 0) {
-            detailPane.destroyContent();
-            let details = display.createDetailsForIndex(index);
-            detailPane.content.add(details, { expand: true });
-            detailPane.open();
-        } else {
-            detailPane.close();
-        }
-    }));
-    return null;
-}
-
 function ResultPane(dash) {
     this._init(dash);
 }
@@ -166,21 +127,12 @@ ResultPane.prototype = {
 
     _init: function(dash) {
         Pane.prototype._init.call(this);
-        this._dash = dash;
-    },
 
-    // Create a display of displayType and pack it into this pane's
-    // content area.  Return the display.
-    packResults: function(displayType) {
-        let resultArea = new ResultArea(displayType);
-
-        createPaneForDetails(this._dash, resultArea.display);
-
-        this.content.add(resultArea.actor, { expand: true });
+        let resultArea = new ResultArea();
+        this.actor.add(resultArea.actor, { expand: true });
         this.connect('open-state-changed', Lang.bind(this, function(pane, isOpen) {
             resultArea.display.resetState();
         }));
-        return resultArea.display;
     }
 };
 
@@ -190,7 +142,7 @@ function SearchEntry() {
 
 SearchEntry.prototype = {
     _init : function() {
-        this.actor = new St.Entry({ name: "searchEntry",
+        this.actor = new St.Entry({ name: 'searchEntry',
                                     hint_text: _("Find") });
         this.entry = this.actor.clutter_text;
 
@@ -198,7 +150,7 @@ SearchEntry.prototype = {
             function() {
                 if (this.isActive())
                     this.actor.set_secondary_icon_from_file(global.imagedir +
-                                                            "close-black.svg");
+                                                            'close-black.svg');
                 else
                     this.actor.set_secondary_icon_from_file(null);
             }));
@@ -230,11 +182,15 @@ SearchEntry.prototype = {
         }
     },
 
-    setPane: function (pane) {
-        this._pane = pane;
-    },
-
     reset: function () {
+        let [x, y, mask] = global.get_pointer();
+        let actor = global.stage.get_actor_at_pos (Clutter.PickMode.REACTIVE,
+                                                   x, y);
+        // this.actor is never hovered directly, only its clutter_text and icon
+        let hovered = this.actor == actor.get_parent();
+
+        this.actor.set_hover(hovered);
+
         this.entry.text = '';
         global.stage.set_key_focus(null);
         this.entry.set_cursor_visible(true);
@@ -257,16 +213,7 @@ SearchEntry.prototype = {
 
     _onCapturedEvent: function(actor, event) {
         let source = event.get_source();
-        let panelEvent = false;
-
-        if (source) {
-            let parent = source;
-            do {
-                if (parent == Main.panel.actor)
-                    break;
-            } while ((parent = parent.get_parent()) != null);
-            panelEvent = (parent != null);
-        }
+        let panelEvent = source && Main.panel.actor.contains(source);
 
         switch (event.type()) {
             case Clutter.EventType.BUTTON_PRESS:
@@ -356,10 +303,21 @@ SearchResult.prototype = {
         this.actor.connect('clicked', Lang.bind(this, this._onResultClicked));
 
         let draggable = DND.makeDraggable(this.actor);
+        draggable.connect('drag-begin',
+                          Lang.bind(this, function() {
+                              Main.overview.beginItemDrag(this);
+                          }));
+        draggable.connect('drag-end',
+                          Lang.bind(this, function() {
+                              Main.overview.endItemDrag(this);
+                          }));
     },
 
     setSelected: function(selected) {
-        this._content.set_style_pseudo_class(selected ? 'selected' : null);
+        if (selected)
+            this._content.add_style_pseudo_class('selected');
+        else
+            this._content.remove_style_pseudo_class('selected');
     },
 
     activate: function() {
@@ -532,7 +490,7 @@ SearchResults.prototype = {
             let meta = this._metaForProvider(provider);
             meta.actor.show();
             meta.resultDisplay.renderResults(providerResults, terms);
-            meta.count.set_text(""+providerResults.length);
+            meta.count.set_text('' + providerResults.length);
         }
 
         this.selectDown(false);
@@ -610,11 +568,11 @@ function MoreLink() {
 
 MoreLink.prototype = {
     _init : function () {
-        this.actor = new St.BoxLayout({ style_class: "more-link",
+        this.actor = new St.BoxLayout({ style_class: 'more-link',
                                         reactive: true });
         this.pane = null;
 
-        this._expander = new St.Bin({ style_class: "more-link-expander" });
+        this._expander = new St.Bin({ style_class: 'more-link-expander' });
         this.actor.add(this._expander, { expand: true, y_fill: false });
     },
 
@@ -630,58 +588,35 @@ MoreLink.prototype = {
     setPane: function (pane) {
         this._pane = pane;
         this._pane.connect('open-state-changed', Lang.bind(this, function(pane, isOpen) {
-            if (!isOpen)
-                this._expander.style_class = 'more-link-expander';
+            if (isOpen)
+                this._expander.add_style_class_name('open');
             else
-                this._expander.style_class = 'more-link-expander open';
+                this._expander.remove_style_class_name('open');
         }));
     }
 };
 
 Signals.addSignalMethods(MoreLink.prototype);
 
-function BackLink() {
-    this._init();
-}
-
-BackLink.prototype = {
-    _init : function () {
-        this.actor = new St.Button({ style_class: "section-header-back",
-                                      reactive: true });
-        this.actor.set_child(new St.Bin({ style_class: "section-header-back-image" }));
-    }
-};
-
 function SectionHeader(title, suppressBrowse) {
     this._init(title, suppressBrowse);
 }
 
 SectionHeader.prototype = {
     _init : function (title, suppressBrowse) {
-        this.actor = new St.Bin({ style_class: "section-header",
+        this.actor = new St.Bin({ style_class: 'section-header',
                                   x_align: St.Align.START,
                                   x_fill: true,
                                   y_fill: true,
                                   reactive: !suppressBrowse });
-        this._innerBox = new St.BoxLayout({ style_class: "section-header-inner" });
+        this._innerBox = new St.BoxLayout({ style_class: 'section-header-inner' });
         this.actor.set_child(this._innerBox);
 
-        this.backLink = new BackLink();
-        this._innerBox.add(this.backLink.actor);
-        this.backLink.actor.hide();
-        this.backLink.actor.connect('clicked', Lang.bind(this, function (actor) {
-            this.emit('back-link-activated');
-        }));
-
-        let textBox = new St.BoxLayout({ style_class: "section-text-content" });
-        this.text = new St.Label({ style_class: "section-title",
+        let textBox = new St.BoxLayout({ style_class: 'section-text-content' });
+        this.text = new St.Label({ style_class: 'section-title',
                                    text: title });
         textBox.add(this.text, { x_align: St.Align.START });
 
-        this.countText = new St.Label({ style_class: "section-count" });
-        textBox.add(this.countText, { expand: true, x_fill: false, x_align: St.Align.END });
-        this.countText.hide();
-
         this._innerBox.add(textBox, { expand: true });
 
         if (!suppressBrowse) {
@@ -695,31 +630,11 @@ SectionHeader.prototype = {
         this.moreLink.activate();
     },
 
-    setTitle : function(title) {
-        this.text.text = title;
-    },
-
-    setBackLinkVisible : function(visible) {
-        if (visible)
-            this.backLink.actor.show();
-        else
-            this.backLink.actor.hide();
-    },
-
     setMoreLinkVisible : function(visible) {
         if (visible)
             this.moreLink.actor.show();
         else
             this.moreLink.actor.hide();
-    },
-
-    setCountText : function(countText) {
-        if (countText == "") {
-            this.countText.hide();
-        } else {
-            this.countText.show();
-            this.countText.text = countText;
-        }
     }
 };
 
@@ -731,14 +646,14 @@ function SearchSectionHeader(title, onClick) {
 
 SearchSectionHeader.prototype = {
     _init : function(title, onClick) {
-        this.actor = new St.Button({ style_class: "dash-search-section-header",
+        this.actor = new St.Button({ style_class: 'dash-search-section-header',
                                       x_fill: true,
                                       y_fill: true });
         let box = new St.BoxLayout();
         this.actor.set_child(box);
-        let titleText = new St.Label({ style_class: "dash-search-section-title",
+        let titleText = new St.Label({ style_class: 'dash-search-section-title',
                                         text: title });
-        this.countText = new St.Label({ style_class: "dash-search-section-count" });
+        this.countText = new St.Label({ style_class: 'dash-search-section-count' });
 
         box.add(titleText);
         box.add(this.countText, { expand: true, x_fill: false, x_align: St.Align.END });
@@ -778,14 +693,14 @@ Dash.prototype = {
         // of the Group actor ends up including the width of its hidden children, so we were getting a reactive object as
         // wide as the details pane that was blocking the clicks to the workspaces underneath it even when the details pane
         // was actually hidden.
-        this.actor = new St.BoxLayout({ name: "dash",
+        this.actor = new St.BoxLayout({ name: 'dash',
                                         vertical: true,
                                         reactive: true });
 
         // The searchArea just holds the entry
-        this.searchArea = new St.BoxLayout({ name: "dashSearchArea",
+        this.searchArea = new St.BoxLayout({ name: 'dashSearchArea',
                                              vertical: true });
-        this.sectionArea = new St.BoxLayout({ name: "dashSections",
+        this.sectionArea = new St.BoxLayout({ name: 'dashSections',
                                                vertical: true });
 
         this.actor.add(this.searchArea);
@@ -884,7 +799,6 @@ Dash.prototype = {
         this._docsSection.header.moreLink.connect('activated', Lang.bind(this, function (link) {
             if (this._moreDocsPane == null) {
                 this._moreDocsPane = new ResultPane(this);
-                this._moreDocsPane.packResults(DOCS);
                 this._addPane(this._moreDocsPane, St.Align.END);
                 link.setPane(this._moreDocsPane);
            }
