@@ -50,6 +50,7 @@ struct _ShellGlobal {
   GjsContext *js_context;
   MutterPlugin *plugin;
   ShellWM *wm;
+  GSettings *settings;
   const char *datadir;
   const char *imagedir;
   const char *userdatadir;
@@ -69,27 +70,20 @@ enum {
 
   PROP_OVERLAY_GROUP,
   PROP_SCREEN,
+  PROP_GDK_SCREEN,
   PROP_SCREEN_WIDTH,
   PROP_SCREEN_HEIGHT,
   PROP_STAGE,
   PROP_WINDOW_GROUP,
   PROP_WINDOW_MANAGER,
+  PROP_SETTINGS,
   PROP_DATADIR,
   PROP_IMAGEDIR,
   PROP_USERDATADIR,
 };
 
-/* Signals */
-enum
-{
-  SCREEN_SIZE_CHANGED,
-  LAST_SIGNAL
-};
-
 G_DEFINE_TYPE(ShellGlobal, shell_global, G_TYPE_OBJECT);
 
-static guint shell_global_signals [LAST_SIGNAL] = { 0 };
-
 static void
 shell_global_set_property(GObject         *object,
                           guint            prop_id,
@@ -120,6 +114,9 @@ shell_global_get_property(GObject         *object,
     case PROP_SCREEN:
       g_value_set_object (value, shell_global_get_screen (global));
       break;
+    case PROP_GDK_SCREEN:
+      g_value_set_object (value, shell_global_get_gdk_screen (global));
+      break;
     case PROP_SCREEN_WIDTH:
       {
         int width, height;
@@ -145,6 +142,9 @@ shell_global_get_property(GObject         *object,
     case PROP_WINDOW_MANAGER:
       g_value_set_object (value, global->wm);
       break;
+    case PROP_SETTINGS:
+      g_value_set_object (value, global->settings);
+      break;
     case PROP_DATADIR:
       g_value_set_string (value, global->datadir);
       break;
@@ -186,6 +186,8 @@ shell_global_init (ShellGlobal *global)
   /* Ensure config dir exists for later use */
   global->userdatadir = g_build_filename (g_get_user_data_dir (), "gnome-shell", NULL);
   g_mkdir_with_parents (global->userdatadir, 0700);
+
+  global->settings = g_settings_new ("org.gnome.shell");
   
   global->grab_notifier = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
   g_signal_connect (global->grab_notifier, "grab-notify", G_CALLBACK (grab_notify), global);
@@ -207,15 +209,6 @@ shell_global_class_init (ShellGlobalClass *klass)
   gobject_class->get_property = shell_global_get_property;
   gobject_class->set_property = shell_global_set_property;
 
-  shell_global_signals[SCREEN_SIZE_CHANGED] =
-    g_signal_new ("screen-size-changed",
-		  G_TYPE_FROM_CLASS (klass),
-		  G_SIGNAL_RUN_LAST,
-		  G_STRUCT_OFFSET (ShellGlobalClass, screen_size_changed),
-		  NULL, NULL,
-		  g_cclosure_marshal_VOID__VOID,
-		  G_TYPE_NONE, 0);
-
   g_object_class_install_property (gobject_class,
                                    PROP_OVERLAY_GROUP,
                                    g_param_spec_object ("overlay-group",
@@ -230,6 +223,15 @@ shell_global_class_init (ShellGlobalClass *klass)
                                                         "Metacity screen object for the shell",
                                                         META_TYPE_SCREEN,
                                                         G_PARAM_READABLE));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_GDK_SCREEN,
+                                   g_param_spec_object ("gdk-screen",
+                                                        "GdkScreen",
+                                                        "Gdk screen object for the shell",
+                                                        GDK_TYPE_SCREEN,
+                                                        G_PARAM_READABLE));
+
   g_object_class_install_property (gobject_class,
                                    PROP_SCREEN_WIDTH,
                                    g_param_spec_int ("screen-width",
@@ -266,6 +268,13 @@ shell_global_class_init (ShellGlobalClass *klass)
                                                         "Window management interface",
                                                         SHELL_TYPE_WM,
                                                         G_PARAM_READABLE));
+  g_object_class_install_property (gobject_class,
+                                   PROP_SETTINGS,
+                                   g_param_spec_object ("settings",
+                                                        "Settings",
+                                                        "GSettings instance for gnome-shell configuration",
+                                                        G_TYPE_SETTINGS,
+                                                        G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_DATADIR,
                                    g_param_spec_string ("datadir",
@@ -425,6 +434,19 @@ shell_global_get_screen (ShellGlobal  *global)
   return mutter_plugin_get_screen (global->plugin);
 }
 
+/**
+ * shell_global_get_gdk_screen:
+ *
+ * Return value: (transfer none): Gdk screen object for the shell
+ */
+GdkScreen *
+shell_global_get_gdk_screen (ShellGlobal *global)
+{
+  g_return_val_if_fail (SHELL_IS_GLOBAL (global), NULL);
+
+  return gdk_screen_get_default ();
+}
+
 /**
  * shell_global_get_windows:
  *
@@ -441,9 +463,10 @@ shell_global_get_windows (ShellGlobal *global)
 }
 
 static gboolean
-update_screen_size (ShellGlobal *global)
+update_screen_size (gpointer data)
 {
   int width, height;
+  ShellGlobal *global = SHELL_GLOBAL (data);
 
   mutter_plugin_query_screen_size (global->plugin, &width, &height);
 
@@ -458,17 +481,6 @@ update_screen_size (ShellGlobal *global)
     clutter_actor_set_size (CLUTTER_ACTOR (global->root_pixmap),
                             width, height);
 
-  return TRUE;
-}
-
-static gboolean
-on_screen_size_changed_cb (gpointer data)
-{
-  ShellGlobal *global = SHELL_GLOBAL (data);
-
-  if (update_screen_size (global))
-    g_signal_emit (G_OBJECT (global), shell_global_signals[SCREEN_SIZE_CHANGED], 0);
-
   return FALSE;
 }
 
@@ -482,7 +494,7 @@ global_stage_notify_width (GObject    *gobject,
   g_object_notify (G_OBJECT (global), "screen-width");
 
   meta_later_add (META_LATER_BEFORE_REDRAW,
-                  on_screen_size_changed_cb,
+                  update_screen_size,
                   global,
                   NULL);
 }
@@ -497,7 +509,7 @@ global_stage_notify_height (GObject    *gobject,
   g_object_notify (G_OBJECT (global), "screen-height");
 
   meta_later_add (META_LATER_BEFORE_REDRAW,
-                  on_screen_size_changed_cb,
+                  update_screen_size,
                   global,
                   NULL);
 }
@@ -1167,7 +1179,7 @@ shell_global_get_monitors (ShellGlobal *global)
 GdkRectangle *
 shell_global_get_primary_monitor (ShellGlobal  *global)
 {
-  GdkScreen *screen = gdk_screen_get_default ();
+  GdkScreen *screen = shell_global_get_gdk_screen (global);
   GdkRectangle rect;
   gint primary = 0;
 
@@ -1286,6 +1298,20 @@ shell_get_event_state (ClutterEvent *event)
   return state & CLUTTER_MODIFIER_MASK;
 }
 
+/**
+ * shell_global_get_settings:
+ * @global: A #ShellGlobal
+ *
+ * Get the global GSettings instance.
+ *
+ * Return value: (transfer none) The GSettings object (transfer none).
+ */
+GSettings *
+shell_global_get_settings (ShellGlobal *global)
+{
+  return global->settings;
+}
+
 static void
 shell_popup_menu_position_func (GtkMenu   *menu,
                                 int       *x,
